package com.cecf.observe;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

public class JSONAnyValueList {
    Logger logger = LoggerFactory.getLogger(JSONAnyValueList.class);
    Map<String, JSONObject> m;

    public JSONAnyValueList(JSONArray arr) {
        this.m = new HashMap<>();
        for (int i = 0; i < arr.length(); i++) {
            JSONObject o = arr.getJSONObject(i);
            this.m.put(o.getString("key"), o.getJSONObject("value"));
        }
    }

    public String getString(String k) {
        JSONObject v = this.m.get(k);
        if (v == null) {
            return "";
        }
        try {
            return v.getString("stringValue");
        } catch (JSONException e) {
            logger.warn("get [{}] from {} failed, not found, [{}]", "stringValue", k, v);
            return "";
        }
    }

    String anyValueToString(JSONObject v) {
        for (Iterator<String> it = v.keys(); it.hasNext(); ) {
            String valueKey = it.next();
            if (valueKey.equals("boolValue")) {
                return v.getBoolean("boolValue") ? "true" : "false";
            }
            if (valueKey.equals("doubleValue")) {
                return Double.toString(v.getDouble("doubleValue"));
            }
            try {
                return v.getString(valueKey);
            } catch (JSONException je) {
                logger.error("{} 不包含String类型的attr值, key:{}", v, valueKey, je);
                return "";
            }
        }
        return "";
    }

    public Map<String, String> toStringMap() {
        Map<String, String> strMap = new HashMap<>();
        for (Map.Entry<String, JSONObject> en : m.entrySet()) {
            String key = en.getKey();
            JSONObject v = en.getValue();
            //"arrayValue":{"values":[{"stringValue":"1YMWWN1N4O"},{"stringValue":"LS4PSXUNUM"},
            // {"stringValue":"66VCHSJNUP"},{"stringValue":"HQTGWGPNH4"},{"stringValue":"L9ECAV7KIM"}]}}
            if (v.has("arrayValue")) {
                List<String> valueStrList = new ArrayList<>();
                JSONObject arrayValueValue = v.getJSONObject("arrayValue");
                JSONArray values = arrayValueValue.getJSONArray("values");
                for (int i = 0; i < values.length(); i++) {
                    JSONObject valueItem = values.getJSONObject(i);
                    valueStrList.add(anyValueToString(valueItem));
                }
                strMap.put(key, String.join(";", valueStrList));
                continue;
            }
            strMap.put(key, anyValueToString(v));
        }
        return strMap;
    }

    public int getInt(String k) {
        JSONObject v = this.m.get(k);
        if (v == null) {
            return 0;
        }
        try {
            return v.getInt("intValue");
        } catch (JSONException e) {
            try {
                String si = v.getString("stringValue");
                return Integer.parseInt(si);
            } catch (JSONException se) {
                logger.warn("get [{}] from {} failed, not found, [{}]", "stringValue", k, v);
                return 0;
            }
        }
    }
}

