package com.cecf.observe;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.ivi.opensource.flinkclickhousesink.ClickHouseSinkConverter;

import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class SpringBootSpanCSVConverter implements ClickHouseSinkConverter<SpringBootSpan> {
    Logger logger = LoggerFactory.getLogger(SpringBootSpanCSVConverter.class);

    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Override
    public String convert(SpringBootSpan ss) {
        StringBuilder builder = new StringBuilder();
        builder.append("(");
        addString(dateFormat.format(ss.Timestamp), builder);
        addString(ss.TraceID, builder);
        addString(ss.SpanID, builder);
        addString(ss.ParentSpanID, builder);
        addString(ss.TraceState, builder);
        addString(ss.SpanName, builder);
        addInt(ss.SpanKind, builder);
        addString(ss.ServiceName, builder);
        addMap(ss.ResourceAttributes, builder);
        addString(ss.ScopeName, builder);
        addString(ss.ScopeVersion, builder);
        addMap(ss.SpanAttributes, builder);
        addLong(ss.Duration, builder);
        addInt(ss.StatusCode, builder);
        addString(ss.StatusMessage, builder);

        builder.append(list2ArrayStr(ss.Events.stream().map(s -> dateFormat.format(s.Timestamp)).collect(Collectors.toList())));
        builder.append(",");
        builder.append(list2ArrayStr(ss.Events.stream().map(s -> s.Name).collect(Collectors.toList())));
        builder.append(",");
        addMapList(ss.Events.stream().map(s -> s.Attributes).collect(Collectors.toList()), builder);

        builder.append(list2ArrayStr(ss.Links.stream().map(s -> s.TraceID).collect(Collectors.toList())));
        builder.append(",");
        builder.append(list2ArrayStr(ss.Links.stream().map(s -> s.SpanID).collect(Collectors.toList())));
        builder.append(",");
        builder.append(list2ArrayStr(ss.Links.stream().map(s -> s.TraceState).collect(Collectors.toList())));
        builder.append(",");
        addMapList(ss.Links.stream().map(s -> s.Attributes).collect(Collectors.toList()), builder);

        addInt(ss.HttpCode, builder);
        addString(ss.HttpMethod, builder);
        addString(ss.HttpURL, builder);

        addString(ss.ContainerID, builder);

        addString(ss.SrcIP, builder);
        addInt(ss.SrcPort, builder);
        addString(ss.TargetIP, builder);
        addInt(ss.TargetPort, builder);

        addString(ss.RPCType, builder);
        addString(ss.RPCName, builder);
        addMap(ss.RPCRequest, builder);
        addInt(ss.RPCResult, builder);

        addString(ss.FuncNamespace, builder);
        addString(ss.FuncName, builder);
        addInt(ss.FuncLineNO, builder);
        addInt(ss.FuncResult, builder);

        addString(ss.DBStatement, builder);
        addString(ss.DBConnectionString, builder);


        builder.append(list2ArrayStr(ss.Exceptions.stream().map(s -> s.Typ).collect(Collectors.toList())));
        builder.append(",");
        builder.append(list2ArrayStr(ss.Exceptions.stream().map(s -> s.Message).collect(Collectors.toList())));
        builder.append(",");
        builder.append(list2ArrayStr(ss.Exceptions.stream().map(s -> s.StackTrace).collect(Collectors.toList())));
        builder.append(",");


        addString(ss.AppName, builder, true);

        builder.append(")");
//        logger.info("SQL语句values:{}", builder.toString());
        return builder.toString();
    }

    String list2ArrayStr(List<String> ss) {
        if (ss == null || ss.isEmpty()) {
            return "[]";
        }
        List<String> ess = ss.stream().map(this::escapeStr).collect(Collectors.toList());
        return "['" + String.join("','", ess) + "']";
    }

    void addMapList(List<Map<String, String>> ms, StringBuilder sb) {
        if (ms == null || ms.isEmpty()) {
            sb.append("[],");
            return;
        }
        sb.append("[");
        for (Map<String, String> m : ms) {
            addMap(m, sb);
        }
        sb.deleteCharAt(sb.length() - 1);
        sb.append("],");
    }

    void addMap(Map<String, String> m, StringBuilder builder) {
        if (m == null || m.isEmpty()) {
            builder.append("{},");
            return;
        }
        builder.append("{");
        for (String key : m.keySet()) {
            addString(key, builder, true);
            builder.append(":");
            addString(m.get(key), builder, true);
            builder.append(",");
        }
        builder.deleteCharAt(builder.length() - 1);
        builder.append("},");
    }


    void addString(String s, StringBuilder builder) {
        if (s == null) {
            builder.append("null,");
            return;
        }
        String es = escapeStr(s);
        builder.append("'");
        builder.append(es);
        builder.append("'");
        builder.append(",");
    }

    void addString(String s, StringBuilder builder, boolean end) {
        addString(s, builder);
        builder.deleteCharAt(builder.length() - 1);
    }

    void addInt(int s, StringBuilder builder) {
        builder.append(String.valueOf(s));
        builder.append(",");
    }

    void addInt(int s, StringBuilder builder, boolean end) {
        builder.append(String.valueOf(s));
    }

    void addLong(long s, StringBuilder builder) {
        builder.append(String.valueOf(s));
        builder.append(",");
    }

    String escapeStr(String s) {
        if (s.contains("'")) {
            return s.replaceAll("'", "''");
        }
        return s;
    }

    void addLong(long s, StringBuilder builder, boolean end) {
        builder.append(String.valueOf(s));
    }
}

